// Ionic Starter App
// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
angular.module('starter', ['ionic', 'ui.bootstrap', 'services', 'ngCordova', 'toaster'])
.run(function($ionicPlatform) {
    $ionicPlatform.ready(function() {
        if (window.cordova && window.cordova.plugins.Keyboard) {
            // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
            // for form inputs)
            cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);

            // Don't remove this line unless you know what you are doing. It stops the viewport
            // from snapping when text inputs are focused. Ionic handles this internally for
            // a much nicer keyboard experience.
            cordova.plugins.Keyboard.disableScroll(true);
        }
        if (window.StatusBar) {
            StatusBar.styleDefault();
        }
    });
})

.config(function($stateProvider, $urlRouterProvider) {

    $stateProvider

    .state('login', {
        url: "/login",
        templateUrl: "templates/login.html"
    })

    .state('signup', {
        url: "/signup",
        templateUrl: "templates/signup.html"
    })

    .state('signin', {
        url: "/signin",
        templateUrl: "templates/signin.html"
    })

    .state('tabs', {
        url: "/tab",
        abstract: true,
        templateUrl: "templates/tabs.html"
    })

    .state('tabs.match', {
        url: "/match",
        views: {
            'match-tab': {
                templateUrl: "templates/states/match.html",
                controller: "MatchCtrl"
            }
        }
    })

    .state('matchDetail', {
        url: "/matchDetail/:matchId",
        params: {
            match: null
        },
        templateUrl: "templates/states/match_detail.html",
        controller: "MatchDetailCtrl"
    })

    .state('tabs.news', {
        url: "/news",
        views: {
            'news-tab': {
                templateUrl: "templates/states/news.html",
                controller: "NewsCtrl"
            }
        }
    })

    .state('tabs.ranking', {
        url: "/ranking",
        views: {
            'ranking-tab': {
                templateUrl: "templates/states/ranking.html",
                controller: "RankingCtrl"
            }
        }
    })

    .state('tabs.redeem', {
        url: "/redeem",
            views: {
            'redeem-tab': {
                templateUrl: "templates/states/redeem.html",
                controller: "RedeemCtrl"
            }
        }
    })

    .state('tabs.slips', {
        url: "/slips",
        views: {
            'slips-tab': {
                templateUrl: "templates/states/slips.html",
                controller: "SlipsCtrl"
            }
        }
    })

    .state('slipsDetail', {
        url: "/slipsDetail/:matchId",
        params: {
            match: null
        },
        templateUrl: "templates/states/match_detail.html",
        controller: "MatchDetailCtrl"
    })

    .state('history', {
        url: "/history",
        templateUrl: "templates/states/redeem_history.html",
        controller: "HistoryCtrl"
    })

    .state('tabs.more', {
        url: "/more",
        views: {
            'more-tab': {
                templateUrl: "templates/states/more.html",
                controller: "MoreCtrl"
            }
        }
    })

    .state('tabs.tips', {
        url: "/tips",
        views: {
            'more-tab': {
                templateUrl: "templates/states/tips.html",
                controller: "MoreCtrl"
            }
        }
    })

    .state('tabs.contact', {
        url: "/contact",
        views: {
            'more-tab': {
                templateUrl: "templates/states/contact.html",
                controller: "MoreCtrl"
            }
        }
    })

    .state('tabs.tutorial', {
        url: "/tutorial",
        views: {
            'more-tab': {
                templateUrl: "templates/states/tutorial.html",
                controller: "MoreCtrl"
            }
        }
    })

    .state('tabs.shop', {
        url: "/shop",
        views: {
            'more-tab': {
                templateUrl: "templates/states/shop.html",
                controller: "MoreCtrl"
            }
        }
    })

    // remove to prevent logged in user from seeing the login page first
    //$urlRouterProvider.otherwise("/login");

    //$httpProvider.interceptors.push('timeoutHttpIntercept');
})

.controller('AppCtrl', function($ionicPlatform, $scope, $rootScope, $timeout, $ionicScrollDelegate, $state, $q, $ionicLoading, $http,
    UserService, $cordovaToast, $ionicModal, HttpService, $cordovaDevice) {

    $ionicPlatform.ready(function() {
        // Check for saved user
        var saved_user = UserService.getUser();
        if ("user" in saved_user && "token_info" in saved_user.user) {
            console.log("found existing user");
            if (saved_user.user.token_info.expiry_time - getCurrentTimestamp() > LOGIN_EXPIRED_PERIOD) {
                console.log("redirect to tabs.match");
                $state.go("tabs.match");
                $scope.updatePushToken();
            } else {
                $scope.exchangeEzBetToken();
            }
        } else {
            console.log("no user found");
            $state.go("login");
        }
    });

    $rootScope.getHeight = function() {
        $timeout(function() {
            $ionicScrollDelegate.resize();
        }, 200)
    };

    $rootScope.activeTabShop = false;

    $scope.updatePushToken = function() {

        /*if (dataaa != "") {
            var deviceId = $cordovaDevice.getUUID();
            var registrationId = dataaa;

            console.log("device id = " + deviceId);
            console.log("push data = " + JSON.stringify(dataaa));

            HttpService.doQuery(
                "POST",
                "http://www.ezbet.online/api/update_push_token",
                {device_id: deviceId, notify_token: registrationId, token: UserService.getEzBetToken()},
                function(result) {
                    console.log("push registered");
                },
                function(error) {
                    console.log("push registered error = " + JSON.stringify(error));
                },
                3
            );
        }*/

        // Only works with android
        // push notification-app
        var push = PushNotification.init({
            android: {
                senderID: "495776853363"
            },
            browser: {
                pushServiceURL: 'http://push.api.phonegap.com/v1/push'
            },
            ios: {
                alert: "true",
                badge: "true",
                sound: "true"
            },
            windows: {}
        });

        push.on('registration', function(data) {
            var deviceId = $cordovaDevice.getUUID();
            var registrationId = data.registrationId;

            console.log("device id = " + deviceId);
            console.log("push data = " + JSON.stringify(data));

            HttpService.doQuery(
                "POST",
                "http://www.ezbet.online/api/update_push_token",
                {device_id: deviceId, notify_token: registrationId, token: UserService.getEzBetToken()},
                function(result) {
                    console.log("push registered");
                },
                function(error) {
                    console.log("push registered error = " + JSON.stringify(error));
                },
                3
            );
        });

        push.on('notification', function(data) {
            console.log("received " + JSON.stringify(data));
            // data.message,
            // data.title,
            // data.count,
            // data.sound,
            // data.image,
            // data.additionalData
        });

        push.on('error', function(e) {
            console.log("error " + JSON.stringify(data));
            // e.message
        });
    };

    /* only works with ios
    FCMPlugin.onTokenRefresh(function(token){
        console.log("token refresh = " + token);
        dataaa = token;
        if (token != null) {
            $scope.updatePushToken();
        }
    });
    FCMPlugin.getToken(function(token){
        console.log("get token = " + token);
        dataaa = token;
        if (token != null) {
            $scope.updatePushToken();
        }
    });*/


    $scope.onTabChanged = function(tab) {
        console.log("tab changed " + tab);
        $scope.$broadcast("onTabChanged", tab);
    };

    $scope.updateActiveTabShop = function (active) {
        $rootScope.activeTabShop = active;
    };

    $scope.goDiamond = function () {
        $rootScope.activeTabShop = true;
        $state.go('tabs.shop');
    };

    $scope.animateIcon = function (number) {
        if (number == 1) {
            $scope.isAnimateLvl = true;
            $timeout(function () {
                $scope.isAnimateLvl = false;
            }, 500)
        }

        if (number == 2) {
            $scope.isAnimateMoney = true;
            $timeout(function () {
                $scope.isAnimateMoney = false;
            }, 500)
        }

        if (number == 3) {
            $scope.isAnimateDiamon = true;
            $timeout(function () {
                $scope.isAnimateDiamon = false;
            }, 500)
        }

    };

    $scope.redirectToLoginPage = function() {
        $state.go("login");
    };

    $scope.rootData = {};
    $scope.globalData = UserService.getUserData();
    $scope.globalData.unreadCount = 0;
    $scope.globalData.notifications = [];

    $ionicModal.fromTemplateUrl('./templates/modals/free-coin.html', {
        scope: $scope,
        animation: 'slide-in-up'
    }).then(function(modal) {
        $scope.modalCoin = modal;
    });
    $scope.openModalCoin = function() {
        $ionicLoading.show();
        $rootScope.updateMissionList();
    };
    $scope.closeModalCoin = function() {
        $scope.modalCoin.hide();
    };
    $scope.$on("updateMissionList", function(event, status) {
        if (status == "succeeded") {
            $scope.modalCoin.show();
            $ionicLoading.hide();
        } else {
            $cordovaToast.show("Không tải được nhiệm vụ", "short", "bottom");
            $ionicLoading.hide();
        }
    });

    $scope.activeTab = function (active) {
        $scope.rootData.tabActive = active;
    };

    $scope.getUnreadNotification = function() {
        HttpService.doQuery(
            "POST",
            "http://www.ezbet.online/api/count_unread_notification",
            {"token": UserService.getEzBetToken()},
            function(result) {
                console.log("success /count_unread_notification: " + JSON.stringify(result));
                if (!isNull(result.data.data) && !isNull(result.data.data.count)) {
                    $scope.globalData.unreadCount = result.data.data.count;
                }
            },
            function(error) {
                console.log("error /count_unread_notification: " + JSON.stringify(error));
            },
            3
        );
    };

    $scope.getNotificationList = function() {
        HttpService.doQuery(
            "POST",
            "http://www.ezbet.online/api/get_notification_list",
            {"token": UserService.getEzBetToken()},
            function(result) {
                console.log("success /get_notification_list: " + JSON.stringify(result));
                if (!isNull(result.data.data)) {
                    $scope.globalData.notifications = [];
                    for (var i = 0; i < result.data.data.length; i++) {
                        if (result.data.data[i].event_type == 1) {
                            var obj = JSON.parse(result.data.data[i].data);
                            $scope.globalData.notifications.push({
                                icon: EventType[result.data.data[i].event_type].icon,
                                title: obj.match_name,
                                message: "Trận đấu đã kết thúc. Xem bạn thắng bao nhiêu!",
                                id: obj.match_id
                            });
                        }
                    }
                }
            },
            function(error) {
                console.log("error /get_notification_list: " + JSON.stringify(error));
            },
            3
        );
    };

    $scope.fromNotificationToMatchDetail = function(match_id) {
        $scope.closeModalNotification();
        $state.go('matchDetail', { matchId : match_id });
    };

    $ionicModal.fromTemplateUrl('./templates/modals/notification.html', {
        scope: $scope,
        animation: 'slide-in-up'
    }).then(function(modal) {
        $scope.modalNotification = modal;
    });
    $scope.openModalNotification = function() {
        $scope.getNotificationList();
        $scope.modalNotification.show();
        $scope.getUnreadNotification();
    };
    $scope.closeModalNotification = function() {
        $scope.modalNotification.hide();
    };

    $ionicModal.fromTemplateUrl('./templates/modals/tos.html', {
        scope: $scope,
        animation: 'slide-in-up'
    }).then(function(modal) {
        $scope.modalTOS = modal;
    });
    $scope.openModalTOS = function() {
        $scope.modalTOS.show();
    };
    $scope.closeModalTOS = function() {
        $scope.modalTOS.hide();
    };

    $scope.$on('$ionicView.loaded', function() {
        //onResume();
    });

    $scope.exchangeEzBetToken = function() {
        var data = {
            token: UserService.getEzBetToken()
        };
        $http({
            method: "POST",
            url: "http://www.ezbet.online/api/exchange_token",
            headers: {
                "Content-Type": "application/x-www-form-urlencoded"
            },
            data: toQueryString(data)
        }).then(function(result) {
            console.log("successssss = " + JSON.stringify(result));
            if ("result" in result.data && result.data.result === ResultCode.SUCCESS) {
                UserService.setEzBetToken(result.data.data.token);
                UserService.setEzBetTokenExpiryTime(result.data.data.expiry_time);
                $scope.updatePushToken();
                $state.go("tabs.match");
            } else if ("result" in result.data && result.data.result === ResultCode.ERROR_TOKEN_EXPIRED) {
                $scope.redirectToLoginPage();
            }
            $ionicLoading.hide();
        }, function(error) {
            console.log("error, cannot exchange token");
        });
    };

    $scope.loginToEzBet = function(backend, token, isAccessToken) {
        console.log("login ezbet, backend = " + backend + ", token = " + token);
        if (backend == AuthBackend.FACEBOOK) {
            HttpService.doQuery(
                "POST",
                "http://www.ezbet.online/auth/register_by_token/facebook",
                {"backend_token": token, "client_type": ClientType.CLIENT_ANDROID, "auth_flag": (isAccessToken ? 0 : 1)},
                function(result) {
                    UserService.setUser(AuthBackend.FACEBOOK, result.data.data);
                    $scope.getUserData();
                    $scope.updatePushToken();
                    $state.go("tabs.match");
                    $ionicLoading.hide();
                },
                function(error) {
                    $ionicLoading.hide();
                    $cordovaToast.show('Lỗi đăng nhập', 'short', 'bottom');
                    console.log("error: " + JSON.stringify(error));
                },
                3
            );
        } else if (backend == AuthBackend.GOOGLE) {
            HttpService.doQuery(
                "POST",
                "http://www.ezbet.online/auth/register_by_token/google-oauth2",
                {"backend_token": token, "client_type": ClientType.CLIENT_ANDROID, "auth_flag": (isAccessToken ? 0 : 1)},
                function(result) {
                    console.log("google, success = " + JSON.stringify(result));
                    console.log("app, set user to service");
                    UserService.setUser(AuthBackend.GOOGLE, result.data.data);
                    $scope.getUserData();
                    $scope.updatePushToken();
                    $state.go("tabs.match");
                    $ionicLoading.hide();
                },
                function(error) {
                    $ionicLoading.hide();
                    $cordovaToast.show('Lỗi đang nhập', 'short', 'bottom');
                    console.log("error: " + JSON.stringify(error));
                },
                3
            );
        }
    };

    // This is the success callback from the login method
    var fbLoginSuccess = function(response) {
        if (!response.authResponse) {
            fbLoginError("Cannot find the authResponse");
            return;
        }

        var authResponse = response.authResponse;

        console.log("lulwut");
        $scope.loginToEzBet(AuthBackend.FACEBOOK, authResponse.accessToken, true);
    };

    // This is the fail callback from the login method
    var fbLoginError = function(error) {
        console.log('fbLoginError' + JSON.stringify(error));
        $ionicLoading.hide();
        $cordovaToast.show('Không đăng nhập được với Facebook', 'short', 'bottom');
    };

    // This method is to get the user profile info from the facebook api
    var getFacebookProfileInfo = function(authResponse) {
        var info = $q.defer();

        facebookConnectPlugin.api('/me?fields=email,name&access_token=' + authResponse.accessToken, null,
            function(response) {
                console.log(response);
                info.resolve(response);
            },
            function(response) {
                console.log(response);
                info.reject(response);
            }
        );
        return info.promise;
    };

    //This method is executed when the user press the "Login with facebook" button
    $scope.facebookLogIn = function() {
        facebookConnectPlugin.getLoginStatus(function(success) {

            $ionicLoading.show({
                template: 'Đang đăng nhập...'
            });

            if (success.status === 'connected') {
                // The user is logged in and has authenticated your app, and response.authResponse supplies
                // the user's ID, a valid access token, a signed request, and the time the access token
                // and signed request each expire

                // Check if we have our user saved
                //var user = UserService.getUser('facebook');

                $scope.loginToEzBet(AuthBackend.FACEBOOK, success.authResponse.accessToken, true);
            } else {
                // If (success.status === 'not_authorized') the user is logged in to Facebook,
                // but has not authenticated your app
                // Else the person is not logged into Facebook,
                // so we're not sure if they are logged into this app or not.

                console.log('getLoginStatus', success.status);

                // Ask the permissions you need. You can learn more about
                // FB permissions here: https://developers.facebook.com/docs/facebook-login/permissions/v2.4
                facebookConnectPlugin.login(['email', 'public_profile'], fbLoginSuccess, fbLoginError);
            }
        });
    };


    $scope.googleLogIn = function() {
        $ionicLoading.show({
            template: 'Đang đăng nhập...'
        });
        window.plugins.googleplus.login(
        {
            'scopes': 'email profile',
            'webClientId': '495776853363-r6d45sa2rvqoasmntsaegmaj1lpogcj5.apps.googleusercontent.com', // optional clientId of your Web application from Credentials settings of your project - On Android, this MUST be included to get an idToken. On iOS, it is not required.
            'offline': true // optional, but requires the webClientId - if set to true the plugin will also return a serverAuthCode, which can be used to grant offline access to a non-Google server
        },
        function (obj) {
            console.log("google = " + JSON.stringify(obj));
            if (obj.accessToken != null && obj.accessToken != "") {
                $scope.loginToEzBet(AuthBackend.GOOGLE, obj.accessToken, true);
            } else {
                $scope.loginToEzBet(AuthBackend.GOOGLE, obj.serverAuthCode, false);
            }
        },
        function (msg) {
            console.log('google login error: ' + JSON.stringify(msg));
            $ionicLoading.hide();
            $cordovaToast.show('Không đăng nhập được với Google', 'short', 'bottom');
        }
        );
    };


    $scope.testLogIn = function() {
        $http({
            method: "GET",
            url: "http://www.ezbet.online/api/test",
            headers: {
                "Content-Type": "application/x-www-form-urlencoded"
            }
        }).then(function(result) {
            if ("result" in result.data && result.data.result === ResultCode.SUCCESS && !isNull(result.data.data)) {
                UserService.setUser(AuthBackend.TEST, {
                    "token_info": {
                        "token": result.data.data.token,
                        "expiry_time": result.data.data.expiry_time
                    }
                });
                $scope.getUserData();
                $scope.updatePushToken();
                $state.go("tabs.match");
            }
        }, function(error) {
            $cordovaToast.show('Cannot test log in', 'short', 'bottom');
            console.log("error: " + JSON.stringify(error));
        });
    };

    $scope.getUserData = function() {
        console.log("get user data, token = " + UserService.getEzBetToken());
        HttpService.doQuery(
            "POST",
            "http://www.ezbet.online/api/get_user_data",
            {version: UserService.getUserDataVersion(), token: UserService.getEzBetToken()},
            function(result) {
                if (!isNull(result.data.data)) {
                    $scope.globalData = result.data.data;
                    UserService.setUserData(result.data.data);
                }
                $ionicLoading.hide();
            },
            function(error) {
                $ionicLoading.hide();
                console.log("get user data error " + JSON.stringify(error));
            },
            3
        );
    };

    $ionicPlatform.on("resume", function() {
        if (UserService.isLoggedIn()) {
            onResume();
        }
    });

    var onResume = function() {
        // login page, do nothing on resume
        //$scope.getUserData();
    };
    console.log("bleah");
});
